# -*- coding: utf-8 -*-
import argparse
import io
import sys
import typing

from config_formatter import ConfigFormatter


def pretty_format_ini(argv: typing.Optional[typing.List[str]] = None) -> int:
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--autofix",
        action="store_true",
        dest="autofix",
        help="Automatically fixes encountered not-pretty-formatted files",
    )

    parser.add_argument("filenames", nargs="*", help="Filenames to fix")
    args = parser.parse_args(argv)

    status = 0

    for ini_file in set(args.filenames):
        with open(ini_file, encoding="utf8") as input_file:
            string_content = input_file.read()

        try:
            formatter = ConfigFormatter()

            pretty_content_str = formatter.prettify(string_content)

            if string_content != pretty_content_str:
                print("File {} is not pretty-formatted".format(ini_file))

                if args.autofix:
                    print("Fixing file {}".format(ini_file))
                    with io.open(ini_file, "w", encoding="UTF-8") as output_file:
                        output_file.write(pretty_content_str)

                status = 1
        except BaseException as e:
            print("Input File {} is not a valid INI file: {}".format(ini_file, e))
            return 1

    return status


if __name__ == "__main__":
    sys.exit(pretty_format_ini())



# -*- coding: utf-8 -*-
import argparse
import io
import sys
import typing

from toml_sort import TomlSort
from toml_sort.tomlsort import CommentConfiguration
from toml_sort.tomlsort import FormattingConfiguration
from toml_sort.tomlsort import SortConfiguration

from language_formatters_pre_commit_hooks.utils import remove_trailing_whitespaces_and_set_new_line_ending


def pretty_format_toml(argv: typing.Optional[typing.List[str]] = None) -> int:
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--autofix",
        action="store_true",
        dest="autofix",
        help="Automatically fixes encountered not-pretty-formatted files",
    )

    parser.add_argument("filenames", nargs="*", help="Filenames to fix")
    args = parser.parse_args(argv)

    status = 0

    for toml_file in set(args.filenames):
        with open(toml_file) as input_file:
            string_content = "".join(input_file.readlines())

        try:
            prettified_content = TomlSort(
                input_toml=string_content,
                comment_config=CommentConfiguration(
                    header=True,
                    footer=True,
                    inline=True,
                    block=True,
                ),
                sort_config=SortConfiguration(tables=True),
                format_config=FormattingConfiguration(
                    spaces_before_inline_comment=2,
                    spaces_indent_inline_array=2,
                    trailing_comma_inline_array=False,
                ),
            ).sorted()

            prettified_content = remove_trailing_whitespaces_and_set_new_line_ending(prettified_content)
            if string_content != prettified_content:
                print("File {} is not pretty-formatted".format(toml_file))

                if args.autofix:
                    print("Fixing file {}".format(toml_file))
                    with io.open(toml_file, "w", encoding="UTF-8") as output_file:
                        output_file.write(prettified_content)

                status = 1
        except BaseException as e:
            print("Input File {} is not a valid TOML file: {}".format(toml_file, e))
            return 1

    return status


if __name__ == "__main__":
    sys.exit(pretty_format_toml())



# -*- coding: utf-8 -*-
import argparse
import sys
import typing

from language_formatters_pre_commit_hooks.pre_conditions import golang_required
from language_formatters_pre_commit_hooks.utils import run_command


def _get_eol_attribute() -> typing.Optional[str]:
    """
    Retrieve eol attribute defined for golang files
    The method will return None in case of any error interacting with git
    """
    status_code, output = run_command("git", "check-attr", "-z", "eol", "--", "filename.go")
    if status_code != 0:
        return None

    try:
        # Expected output: "filename.go\0eol\0lf\0"
        _, _, eol, _ = output.split("\0")
        return eol
    except:  # noqa: E722 (allow usage of bare 'except')
        print(
            "`git check-attr` output is not consistent to `<filename>\0<key>\0<value>\0` format: {output}".format(
                output=output,
            ),
            file=sys.stderr,
        )
        return None


@golang_required
def pretty_format_golang(argv: typing.Optional[typing.List[str]] = None) -> int:
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--autofix",
        action="store_true",
        dest="autofix",
        help="Automatically fixes encountered not-pretty-formatted files",
    )

    parser.add_argument("filenames", nargs="*", help="Filenames to fix")
    args = parser.parse_args(argv)

    cmd_args = ["gofmt", "-l"]
    if args.autofix:
        cmd_args.append("-w")
    status, output = run_command(*(cmd_args + args.filenames))

    if status != 0:  # pragma: no cover
        print(output)
        return 1

    status = 0
    if output:
        status = 1
        print(
            "{}: {}".format(
                "The following files have been fixed by gofmt" if args.autofix else "The following files are not properly formatted",
                ", ".join(output.splitlines()),
            ),
        )
        if sys.platform == "win32":  # pragma: no cover
            eol_attribute = _get_eol_attribute()
            if eol_attribute and eol_attribute != "lf":
                print(
                    "Hint: gofmt uses LF (aka `\\n`) as new line, but on Windows the default new line is CRLF (aka `\\r\\n`). "
                    "You might want to ensure that go files are forced to use LF via `.gitattributes`. "
                    "Example: https://github.com/macisamuele/language-formatters-pre-commit-hooks/commit/53f27fda02ead5b1b9b6a9bbd9c36bb66d229887",  # noqa: E501
                    file=sys.stderr,
                )

    return status


if __name__ == "__main__":
    sys.exit(pretty_format_golang())



# -*- coding: utf-8 -*-
import pkg_resources


__version__ = pkg_resources.get_distribution("language_formatters_pre_commit_hooks").version


def _get_default_version(tool_name: str) -> str:  # pragma: no cover
    """
    Read tool_name default version.
    The method is intended to be used only from language_formatters_pre_commit_hooks modules
    """
    try:
        with open(
            pkg_resources.resource_filename(
                "language_formatters_pre_commit_hooks",
                "{tool_name}.version".format(tool_name=tool_name),
            )
        ) as f:
            return f.readline().split()[0]
    except:  # noqa: E722 (allow usage of bare 'except')
        raise RuntimeError("No default version found for {tool_name}".format(tool_name=tool_name))



# -*- coding: utf-8 -*-
import os
import shutil
import subprocess  # nosec B404 B603
import sys
import tempfile
import typing
from urllib.parse import urlparse

import requests


def run_command(*command: str) -> typing.Tuple[int, str]:
    print("[cwd={cwd}] Run command: {command}".format(command=command, cwd=os.getcwd()), file=sys.stderr)
    return_code, output = 1, ""
    try:
        return_code, output = (
            0,
            subprocess.check_output(  # nosec: disable=B603
                command,
                stderr=subprocess.STDOUT,
            ).decode("utf-8"),
        )
    except subprocess.CalledProcessError as e:
        return_code, output = e.returncode, e.output.decode("utf-8")
    print("[return_code={return_code}] | {output}".format(return_code=return_code, output=output), file=sys.stderr)
    return return_code, output


def _base_directory() -> str:
    # Extracted from pre-commit code:
    # https://github.com/pre-commit/pre-commit/blob/master/pre_commit/store.py
    return os.path.realpath(
        os.environ.get(str("PRE_COMMIT_HOME"))
        or os.path.join(
            os.environ.get(str("XDG_CACHE_HOME")) or os.path.expanduser("~/.cache"),
            "pre-commit",
        ),
    )


def download_url(url: str, file_name: typing.Optional[str] = None) -> str:
    base_directory = _base_directory()

    final_file = os.path.join(
        base_directory,
        file_name or os.path.basename(urlparse(url).path),
    )

    if os.path.exists(final_file):
        return final_file

    if not os.path.exists(base_directory):  # pragma: no cover
        # If the base directory is not present we should create it.
        # This is needed to allow the tool to run if invoked via
        # command line, but it should never be possible if invoked
        # via `pre-commit` as it would ensure that the directories
        # are present
        print("Unexisting base directory ({base_directory}). Creating it".format(base_directory=base_directory), file=sys.stderr)
        os.makedirs(base_directory)

    print("Downloading {url}".format(url=url), file=sys.stderr)
    r = requests.get(url, stream=True)
    r.raise_for_status()
    with tempfile.NamedTemporaryFile(dir=base_directory, delete=False) as tmp_file:  # Not delete because we're renaming it
        tmp_file_name = tmp_file.name
        shutil.copyfileobj(r.raw, tmp_file)
        tmp_file.flush()
        os.fsync(tmp_file.fileno())

    os.rename(tmp_file_name, final_file)

    return final_file


def remove_trailing_whitespaces_and_set_new_line_ending(string: str) -> str:
    return "{content}\n".format(
        content="\n".join(line.rstrip() for line in string.splitlines()).rstrip(),
    )



# -*- coding: utf-8 -*-
import argparse
import io
import re
import sys
import typing
from sys import maxsize

from ruamel.yaml import YAML


def _process_single_document(document: str, yaml: YAML) -> str:
    """Pretty format one YAML document.

    This is needed in order to prevent `ruamel.yaml` to interfere with documents that have primitive types on the document root.
    For more context check https://github.com/macisamuele/language-formatters-pre-commit-hooks/pull/1

    Args:
        document (str): Original document content.
        yaml: YAML library instance.
    Returns:
        Pretty-formatted content (str).
    """
    content = yaml.load(document)
    if isinstance(content, (list, dict)):
        pretty_output = io.StringIO()
        yaml.dump(content, pretty_output)
        return pretty_output.getvalue()
    else:
        # do not disturb primitive content (unstructured text)
        return str(document)


def pretty_format_yaml(argv: typing.Optional[typing.List[str]] = None) -> int:
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--autofix",
        action="store_true",
        dest="autofix",
        help="Automatically fixes encountered not-pretty-formatted files",
    )
    parser.add_argument(
        "--indent",
        type=int,
        default="2",
        help="The number of indent spaces or a string to be used as delimiter for indentation level e.g. 4 or '\t' (Default: 2)",
    )
    parser.add_argument(
        "--preserve-quotes",
        action="store_true",
        dest="preserve_quotes",
        help="Keep existing string quoting",
    )
    parser.add_argument(
        "--line-width",
        type=int,
        default=maxsize,
        dest="line_width",
        help=(
            "Max line length on the generated file. NOTE: As far as we attempt to"
            " enforce the limit we cannot guarantee that it is always possible"
        ),
    )

    parser.add_argument("filenames", nargs="*", help="Filenames to fix")
    args = parser.parse_args(argv)

    status = 0

    yaml = YAML()
    yaml.indent = args.indent
    yaml.preserve_quotes = args.preserve_quotes
    # Prevent ruamel.yaml to wrap yaml lines
    yaml.width = args.line_width  # type: ignore  # mypy recognise yaml.width as None

    separator = "---\n"

    for yaml_file in set(args.filenames):
        with open(yaml_file, encoding="utf8") as input_file:
            string_content = "".join(input_file.readlines())

        # Split multi-document file into individual documents
        #
        # Not using yaml.load_all() because it reformats primitive (non-YAML) content. It removes
        # newline characters.
        separator_pattern = r"^---\s*\n"
        original_docs = re.split(separator_pattern, string_content, flags=re.MULTILINE)

        # A valid multi-document YAML file might starts with the separator.
        # In this case the first document of original docs will be empty and should not be consdered
        if string_content.startswith("---"):
            original_docs = original_docs[1:]

        pretty_docs = []

        try:
            for doc in original_docs:
                content = _process_single_document(doc, yaml)
                if content is not None:
                    pretty_docs.append(content)

            # Start multi-doc file with separator
            pretty_content = "" if len(pretty_docs) == 1 else separator
            pretty_content += separator.join(pretty_docs)

            if string_content != pretty_content:
                print("File {} is not pretty-formatted".format(yaml_file))

                if args.autofix:
                    print("Fixing file {}".format(yaml_file))
                    with io.open(yaml_file, "w", encoding="UTF-8") as output_file:
                        output_file.write(str(pretty_content))

                status = 1
        except BaseException as e:  # pragma: no cover
            print(
                "Input File {} is not a valid YAML file, consider using check-yaml: {}".format(yaml_file, e),
            )
            return 1

    return status


if __name__ == "__main__":
    sys.exit(pretty_format_yaml())



# -*- coding: utf-8 -*-
import argparse
import sys
import typing

from language_formatters_pre_commit_hooks import _get_default_version
from language_formatters_pre_commit_hooks.pre_conditions import java_required
from language_formatters_pre_commit_hooks.utils import download_url
from language_formatters_pre_commit_hooks.utils import run_command


def _download_kotlin_formatter_jar(version: str) -> str:  # pragma: no cover
    def get_url(_version: str) -> str:
        # Links extracted from https://github.com/pinterest/ktlint/
        return "https://github.com/pinterest/ktlint/releases/download/{version}/ktlint".format(
            version=_version,
        )

    url_to_download = get_url(version)
    try:
        return download_url(get_url(version), "ktlint{version}.jar".format(version=version))
    except:  # noqa: E722 (allow usage of bare 'except')
        raise RuntimeError(
            "Failed to download {url}. Probably the requested version, {version}, is "
            "not valid or you have some network issue.".format(
                url=url_to_download,
                version=version,
            ),
        )


def _fix_paths(paths: typing.Iterable[str]) -> typing.Iterable[str]:
    # Starting from KTLint 0.41.0 paths cannot contain backward slashes as path separator
    # Odd enough the error messages reported by KTLint contain `\` :(
    for path in paths:
        yield path.replace("\\", "/")


@java_required
def pretty_format_kotlin(argv: typing.Optional[typing.List[str]] = None) -> int:
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--autofix",
        action="store_true",
        dest="autofix",
        help="Automatically fixes encountered not-pretty-formatted files",
    )
    parser.add_argument(
        "--ktlint-version",
        dest="ktlint_version",
        default=_get_default_version("ktlint"),
        help="KTLint version to use (default %(default)s)",
    )

    parser.add_argument("filenames", nargs="*", help="Filenames to fix")
    args = parser.parse_args(argv)

    ktlint_jar = _download_kotlin_formatter_jar(
        args.ktlint_version,
    )

    jvm_args = ["--add-opens", "java.base/java.lang=ALL-UNNAMED"]

    # ktlint does not return exit-code!=0 if we're formatting them.
    # To workaround this limitation we do run ktlint in check mode only,
    # which provides the expected exit status and we run it again in format
    # mode if autofix flag is enabled
    check_status, check_output = run_command(
        "java", *jvm_args, "-jar", ktlint_jar, "--log-level", "debug", "--relative", "--", *_fix_paths(args.filenames)
    )

    not_pretty_formatted_files: typing.Set[str] = set()
    if check_status != 0:
        not_pretty_formatted_files.update(line.split(":", 1)[0] for line in check_output.splitlines())

        if args.autofix:
            print("Running ktlint format on {}".format(not_pretty_formatted_files))
            run_command(
                "java",
                *jvm_args,
                "-jar",
                ktlint_jar,
                "--log-level",
                "debug",
                "--relative",
                "--format",
                "--",
                *_fix_paths(not_pretty_formatted_files),
            )

    status = 0
    if not_pretty_formatted_files:
        status = 1
        print(
            "{}: {}".format(
                "The following files have been fixed by ktlint" if args.autofix else "The following files are not properly formatted",
                ", ".join(sorted(not_pretty_formatted_files)),
            ),
        )

    return status


if __name__ == "__main__":
    sys.exit(pretty_format_kotlin())



# -*- coding: utf-8 -*-
import argparse
import sys
import typing

from language_formatters_pre_commit_hooks.pre_conditions import rust_required
from language_formatters_pre_commit_hooks.utils import run_command


@rust_required
def pretty_format_rust(argv: typing.Optional[typing.List[str]] = None) -> int:
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--autofix",
        action="store_true",
        dest="autofix",
        help="Automatically fixes encountered not-pretty-formatted files",
    )

    parser.add_argument("filenames", nargs="*", help="Filenames to fix")
    args = parser.parse_args(argv)

    # Check
    status_code, output = run_command("cargo", "fmt", "--", "--check", *args.filenames)
    not_well_formatted_files = sorted(line.split()[2] for line in output.splitlines() if line.startswith("Diff in "))
    if not_well_formatted_files:
        print(
            "{}: {}".format(
                "The following files have been fixed by cargo format" if args.autofix else "The following files are not properly formatted",
                ", ".join(not_well_formatted_files),
            ),
        )
        if args.autofix:
            run_command("cargo", "fmt", "--", *not_well_formatted_files)
    elif status_code != 0:
        print("Detected not valid rust source files among {}".format("\n".join(sorted(args.filenames))))

    return 1 if status_code != 0 or not_well_formatted_files else 0


if __name__ == "__main__":
    sys.exit(pretty_format_rust())



# -*- coding: utf-8 -*-
import re
import typing
from functools import wraps

from packaging.version import Version

from language_formatters_pre_commit_hooks.utils import run_command


F = typing.TypeVar("F", bound=typing.Callable[..., int])


def _is_command_success(
    *command_args: str,
    output_should_match: typing.Optional[typing.Callable[[str], bool]] = None,
) -> bool:
    exit_status, output = run_command(*command_args)
    return exit_status == 0 and (output_should_match is None or output_should_match(output))


class ToolNotInstalled(RuntimeError):
    def __init__(self, tool_name: str, download_install_url: str) -> None:
        self.tool_name = tool_name
        self.download_install_url = download_install_url

    def __str__(self) -> str:
        return str(
            "{tool_name} is required to run this pre-commit hook.\n"
            "Make sure that you have it installed and available on your path.\n"
            "Download/Install URL: {download_install_url}".format(
                tool_name=self.tool_name,
                download_install_url=self.download_install_url,
            )
        )


class _ToolRequired:
    def __init__(
        self,
        tool_name: str,
        check_command: typing.Callable[[typing.Optional[typing.Mapping[str, typing.Any]]], bool],
        download_install_url: str,
        extras: typing.Optional[typing.Mapping[str, typing.Any]] = None,
    ) -> None:
        self.tool_name = tool_name
        self.check_command = check_command
        self.download_install_url = download_install_url
        self.extras = extras

    def is_tool_installed(self) -> bool:
        return self.check_command(self.extras)

    def assert_tool_installed(self) -> None:
        if not self.is_tool_installed():
            raise ToolNotInstalled(
                tool_name=self.tool_name,
                download_install_url=self.download_install_url,
            )

    def __call__(self, f: F) -> F:
        @wraps(f)
        def wrapper(*args: typing.Any, **kwargs: typing.Any) -> int:
            self.assert_tool_installed()
            return f(*args, **kwargs)

        return wrapper  # type: ignore


java_required = _ToolRequired(
    tool_name="JRE",
    check_command=lambda _: _is_command_success("java", "-version"),
    download_install_url="https://www.java.com/en/download/",
)

golang_required = _ToolRequired(
    tool_name="golang/gofmt",
    check_command=lambda _: _is_command_success("go", "version"),
    download_install_url="https://golang.org/doc/install#download",
)


rust_required = _ToolRequired(
    tool_name="rustfmt",
    check_command=(lambda _: _is_command_success("cargo", "fmt", "--", "--version")),
    download_install_url="https://github.com/rust-lang-nursery/rustfmt#quick-start",
)


class UnableToVerifyJDKVersion(RuntimeError):
    def __str__(self) -> str:
        return "Unable to verify the JDK version"  # pragma: no cover


def get_jdk_version() -> Version:
    """
    Extract the version of the JDK accessible by the tool.

    :raises UnableToVerifyJDKVersion: if it was not possible to gather the JDK version
        This includes the case of `java` binary is not found in the path.
    """
    _, output = run_command("java", "-XshowSettings:properties", "-version")
    try:
        java_property_line = next(line for line in output.splitlines() if re.match(r"^\s+java.version\s+=\s+[^s]+$", line))
        return Version(java_property_line.split()[-1])
    except Exception as e:
        raise UnableToVerifyJDKVersion() from e


def assert_min_jdk_version(version: Version) -> None:
    """
    Ensure that the version of the accessible JDK is at least at the provided version.

    :raises UnableToVerifyJDKVersion: if it was not possible to gather the JDK version
        This includes the case of `java` binary is not found in the path.
    :raises ToolNotInstalled: if `java` binary is found in the path but the JDK version does not
        respect the min version requirement
    """
    _ToolRequired(
        tool_name=f"JRE: min version {version}",
        check_command=lambda extras: bool(extras and get_jdk_version() >= extras["min_sdk"]),
        download_install_url="https://www.java.com/en/download/",
        extras={"min_sdk": version},
    ).assert_tool_installed()


def assert_max_jdk_version(version: Version, *, inclusive: bool = False) -> None:
    """
    Ensure that the version of the accessible JDK is at most at the provided version.
    The inclusive parameter allows us to include or exclude the specified version:
    * if `inclusive=True` then `get_jdk_version() <= version` is evaluated
    * if `inclusive=False` then `get_jdk_version() < version` is evaluated
      NOTE: The missing `=` in the operation

    :raises UnableToVerifyJDKVersion: if it was not possible to gather the JDK version
        This includes the case of `java` binary is not found in the path.
    :raises ToolNotInstalled: if `java` binary is found in the path but the JDK version does not
        respect the max version requirement
    """
    if inclusive:
        tool_name = f"JRE: version <= {version}"
    else:
        tool_name = f"JRE: version < {version}"

    _ToolRequired(
        tool_name=tool_name,
        check_command=lambda extras: bool(
            extras and (get_jdk_version() <= extras["max_sdk"] if inclusive else get_jdk_version() < extras["max_sdk"])
        ),
        download_install_url="https://www.java.com/en/download/",
        extras={"max_sdk": version},
    ).assert_tool_installed()



# -*- coding: utf-8 -*-
from setuptools import setup


setup(
    extras_require={
        ":python_version<'3.5'": ["typing"],
    },
    entry_points={
        "console_scripts": [
            "pretty-format-golang = language_formatters_pre_commit_hooks.pretty_format_golang:pretty_format_golang",
            "pretty-format-java = language_formatters_pre_commit_hooks.pretty_format_java:pretty_format_java",
            "pretty-format-kotlin = language_formatters_pre_commit_hooks.pretty_format_kotlin:pretty_format_kotlin",
            "pretty-format-ini = language_formatters_pre_commit_hooks.pretty_format_ini:pretty_format_ini",
            "pretty-format-rust = language_formatters_pre_commit_hooks.pretty_format_rust:pretty_format_rust",
            "pretty-format-toml = language_formatters_pre_commit_hooks.pretty_format_toml:pretty_format_toml",
            "pretty-format-yaml = language_formatters_pre_commit_hooks.pretty_format_yaml:pretty_format_yaml",
        ],
    },
)



# -*- coding: utf-8 -*-
import argparse
import sys
import typing

import requests
from packaging.version import Version

from language_formatters_pre_commit_hooks import _get_default_version
from language_formatters_pre_commit_hooks.pre_conditions import assert_max_jdk_version
from language_formatters_pre_commit_hooks.pre_conditions import java_required
from language_formatters_pre_commit_hooks.utils import download_url
from language_formatters_pre_commit_hooks.utils import run_command


def _download_google_java_formatter_jar(version: str) -> str:  # pragma: no cover
    def get_urls(_version: str) -> typing.List[str]:
        # Links extracted from https://github.com/google/google-java-format/
        return [
            "https://github.com/google/google-java-format/releases/download/"
            "v{version}/google-java-format-{version}-all-deps.jar".format(
                version=_version,
            ),
            # Versions older than 1.10 have a different template
            "https://github.com/google/google-java-format/releases/download/"
            "google-java-format-{version}/google-java-format-{version}-all-deps.jar".format(
                version=_version,
            ),
        ]

    possible_urls = get_urls(version)
    try:
        for url_to_download in possible_urls:
            try:
                return download_url(url_to_download, "google-java-formatter{version}.jar".format(version=version))
            except requests.HTTPError as e:
                if e.response.status_code != 404:
                    # If the url was not found then move forward with the next links
                    raise

        raise RuntimeError("Failed to load any of the provided links")
    except:  # noqa: E722 (allow usage of bare 'except')
        raise RuntimeError(
            "Failed to download any of: {urls}. Probably the requested version, "
            "{version}, is not valid or you have some network issue.".format(
                urls=", ".join(possible_urls),
                version=version,
            ),
        )


@java_required
def pretty_format_java(argv: typing.Optional[typing.List[str]] = None) -> int:
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--autofix",
        action="store_true",
        dest="autofix",
        help="Automatically fixes encountered not-pretty-formatted files",
    )
    parser.add_argument(
        "--google-java-formatter-version",
        dest="google_java_formatter_version",
        default=_get_default_version("google_java_formatter"),
        help="Google Java Formatter version to use (default %(default)s)",
    )
    parser.add_argument(
        "--aosp",
        action="store_true",
        dest="aosp",
        help="Formats Java code into AOSP format",
    )

    parser.add_argument("filenames", nargs="*", help="Filenames to fix")
    args = parser.parse_args(argv)

    # Google Java Formatter 1.10+ does support Java 16+, before that version
    # the tool can only be executed on Java up to version 15.
    # Context: https://github.com/google/google-java-format/releases/tag/v1.10.0
    if Version(args.google_java_formatter_version) <= Version("1.9"):
        assert_max_jdk_version(Version("16.0"), inclusive=False)  # pragma: no cover

    google_java_formatter_jar = _download_google_java_formatter_jar(
        args.google_java_formatter_version,
    )

    cmd_args = [
        "java",
        # export JDK internal classes for Java 16+
        "--add-exports",
        "jdk.compiler/com.sun.tools.javac.api=ALL-UNNAMED",
        "--add-exports",
        "jdk.compiler/com.sun.tools.javac.file=ALL-UNNAMED",
        "--add-exports",
        "jdk.compiler/com.sun.tools.javac.parser=ALL-UNNAMED",
        "--add-exports",
        "jdk.compiler/com.sun.tools.javac.tree=ALL-UNNAMED",
        "--add-exports",
        "jdk.compiler/com.sun.tools.javac.util=ALL-UNNAMED",
        "-jar",
        google_java_formatter_jar,
        "--set-exit-if-changed",
    ]
    if args.aosp:  # pragma: no cover
        cmd_args.append("--aosp")
    if args.autofix:
        cmd_args.append("--replace")
    else:
        cmd_args.append("--dry-run")
    status, output = run_command(*(cmd_args + args.filenames))

    if output:
        print(
            "{}: {}".format(
                "The following files have been fixed by google-java-formatter"
                if args.autofix
                else "The following files are not properly formatted",  # noqa
                ", ".join(output.splitlines()),
            ),
        )

    return 0 if status == 0 else 1


if __name__ == "__main__":
    sys.exit(pretty_format_java())
